# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-01-25 17:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0007_auto_20160111_1844'),
    ]

    operations = [
        migrations.CreateModel(
            name='RSAKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.TextField()),
            ],
        ),
    ]
